/**************************************************************************************************
	$Header: /pub/cvsroot/mydns/src/mydns/cache.h,v 1.10 2002/07/29 07:11:08 bboy Exp $

	Copyright (C) 2002  Don Moore <bboy@bboy.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at Your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**************************************************************************************************/

#ifndef _CACHE_H
#define _CACHE_H

/* The actual number of slots specified for `nodes' is the limit multiplied by this number.
	A higher value will yield greater speed (due to fewer collisions) but will use more memory. */
#define CACHE_SLOT_MULTIPLIER	3


typedef struct _cnode						/* A cache node */
{
	u_int32_t	hash;							/* Hash value, for lookup when deleting nodes */
	u_int32_t	zone;							/* Zone for record */
	dns_qtype_t	type;							/* Record type */
	u_char		name[DNS_MAXNAMELEN+1];	/* The name to look up */
	size_t		namelen;						/* strlen(name) */
	void			*data;						/* SOA or RR record (depending on `type') */
	time_t		expire;						/* Time after which this node should expire */

	struct _cnode *next_node;				/* Pointer to next node in bucket */
	struct _cnode *mru_prev, *mru_next;	/* Pointer to next/prev node in MRU/LRU list */
} CNODE;



typedef struct _cache						/* A cache */
{
	size_t		limit;						/* Max number of nodes allowed (via `cache-size') */
	size_t		count;						/* Number of used nodes in the cache */

	time_t		expire;						/* Expiration seconds for cached items */

	u_int32_t	questions;					/* Total number of questions */
	u_int32_t	hits, misses;				/* Total number of hits/misses for the cache */
	u_int32_t	in, out;						/* Cache entries in and out */
	u_int32_t	expired, removed;			/* Cache entries expired and removed */

	size_t		slots;						/* Number of slots in `nodes' array */
	CNODE			**nodes;						/* Array of nodes */

	CNODE			*mru_head, *mru_tail;	/* Most/Least recently used (MRU/LRU) nodes (head/tail) */
} CACHE;


extern CACHE *Cache;							/* Our cache */

extern char *cache_status(void);
extern void cache_init(), cache_empty(), cache_cleanup();
extern void *cache_find(u_int32_t, dns_qtype_t, u_char *, size_t);

#endif /* _CACHE_H */

/* vi:set ts=3: */
